indexing
   description: "This creates the color canvas that"
   "is in the gradient_editor."
   status: "Free according to the GPL license (see COPYING)"
   date: "$Date$"
   revision: "$Revision$"
   author: "Mark Arrasmith <arrasmith@math.twsu.edu> and Miron Brezuleanu"
   

class GRADIENT_EDITOR_CANVAS
   
inherit
   GTK_DRAWING_AREA
      redefine
	 make, 	
	 expose_handler,
	 button_press_handler
      end
   EXG_CONSTANTS
   STARTUP
   
creation
   make
   
feature -- commands
   
   make is
	 -- sets up the canvas
      do
	 Precursor
	 set_size(40, 200)
	 activate_event_handling
      end

   update is
	 -- the command that ensures complete repainting
      do
	 reset_gradient
	 expose_handler(Void)
      end
    
   expose_handler(r: GDK_RECTANGLE) is
	 -- draws the colors
      local
	 i, j: INTEGER
	 gc: GDK_GC
	 h: INTEGER
	 step: DOUBLE
      do
	 from
	    i := colors.palette.lower
	    !!gc.make(current)
	    h := allocation.height // colors.palette.count
	 until
	    i > colors.palette.upper
	 loop
	    gc.set_foreground(colors.palette.item(i))
	    draw_rectangle(gc, True, 
			   0, (i - colors.palette.lower) * h,
			   allocation.width // 2, h)
	    i := i + 1
	 end
	 if gradient = Void then
	    gradient := colors.gradient
	 end
	 from
	    i := 0
	    step := gradient.count / allocation.height
	 until	 
	    i = allocation.height
	 loop
	    gc.set_foreground(gradient.item((i*step).floor+1))
	    draw_line(gc, allocation.width // 2, i,
		      allocation.width - 1, i)
	    i := i + 1
	 end
	 gc.destroy
	 event_result := True
      end
   
   button_press_handler(be: GDK_EVENT_BUTTON) is
	 -- will handle button clicks, updating colors
      
	 -- the left button will edit the available colors
      local
	 color_no: INTEGER
	 ecsm: EXG_COLOR_SELECTION_MODAL
      do
	 color_no := be.y.rounded // 
	    (allocation.height // colors.palette.count) + 1
	 if color_no >= colors.palette.lower 
	    and color_no <= colors.palette.upper then
	    if be.button = 1 then
	       !!ecsm.make
	       ecsm.set_title("Select a color")
	       ecsm.set_color(colors.palette.item(color_no))
	       ecsm.run
	       if equal(ecsm.answer, "ok") then
		  colors.palette.put(ecsm.color, color_no)
		  gradient := colors.gradient
		  expose_handler(Void)
	       end
	    end
	 end
	 event_result := True
      end
   
   set_dialog(nv: GRADIENT_EDITOR) is
	 -- setter command for `dialog'
      do
	 dialog := nv
      end
   
   reset_gradient is
	 -- a trivial command to force the recalculation of the gradient
      do
	 gradient := Void
      end
   
feature -- queries and attributes
   
   dialog: GRADIENT_EDITOR
   gradient: COLLECTION[GDK_COLOR]
   
end
   
